<?xml version="1.0"?>
<!-- ============================================================ -->
<!--                                                              -->
<!-- Copyright (c) 2004 RenderX, Inc. All Rights Reserved.        -->
<!-- This software is provided as a sample for evaluation         -->
<!-- of XEP formatter only. RenderX declines every liability      -->
<!-- for negative effects of its use for other purposes.          -->
<!--                                                              -->
<!-- Author: Alexander Peshkov <peshkov@renderx.com>              -->
<!-- Revised: Nikolai Grigoriev <grig@renderx.com>                -->
<!--                                                              -->
<!-- ============================================================ -->

<!-- ============================================================ -->
<!-- This stylesheet turns a ordinary document with sequential    -->
<!-- pages into a landscape-oriented document.                    --> 
<!-- ============================================================ -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xep="http://www.renderx.com/XEP/xep"
                version="1.0">
                
  <!-- ===================================================== -->
  <!-- Global variables: page width and height; total pages  -->
  <!-- ===================================================== -->
  <xsl:variable name="page-width" select="/xep:document/xep:page[1]/@width"/>
  <xsl:variable name="page-height" select="/xep:document/xep:page[1]/@height"/>
  <xsl:variable name="total-pages" select="count(/xep:document/xep:page)"/>
  

  <!-- ================================================== -->
  <!-- Top-level template: copy the document node,        -->
  <!-- then start a recursive procedure to reorder pages  -->
  <!-- ================================================== -->
  <xsl:template match="xep:document">
    <xep:document>
      <xsl:apply-templates select="@*"/>
      <xsl:apply-templates select="xep:internal-bookmark"/>
      <xsl:apply-templates select="xep:external-bookmark"/>
      
      <xsl:call-template name="make-sheet">
        <xsl:with-param name="sheet-number" select="1"/>
        <xsl:with-param name="first-page" select="1"/>
      </xsl:call-template>
    </xep:document>
  </xsl:template>


  <!-- ================================================ -->
  <!-- Draw a complete sheet of 4 pages, then call      -->
  <!-- the same template recursively for the next sheet -->
  <!-- ================================================ -->
  <xsl:template name="make-sheet">  
    <xsl:param name="sheet-number"/>
    <xsl:param name="first-page"/>

    <xsl:if test="$first-page &lt;= $total-pages"> 
      <!-- The first page -->
      <xsl:call-template name="draw-page">
        <xsl:with-param name="sheet-number" select="$sheet-number"/>
        <xsl:with-param name="left-page" select="$first-page"/>
        <xsl:with-param name="right-page" select="$first-page + 2"/>
      </xsl:call-template>

      <!-- The second page -->
      <xsl:call-template name="draw-page">
        <xsl:with-param name="sheet-number" select="$sheet-number + 1"/>
        <xsl:with-param name="left-page" select="$first-page + 1"/>
        <xsl:with-param name="right-page" select="$first-page + 3"/>
      </xsl:call-template>

      <!-- Recursion -->
      <xsl:call-template name="make-sheet">
        <xsl:with-param name="sheet-number" select="$sheet-number + 2"/>
        <xsl:with-param name="first-page" select="$first-page + 4"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>
    
  <!-- ================================================================ -->
  <!-- Draw a single page in the output (containing two original pages) -->
  <!-- ================================================================ -->
  <xsl:template name="draw-page">  
    <xsl:param name="sheet-number"/>
    <xsl:param name="left-page"/>
    <xsl:param name="right-page"/>   
      
    <xep:page width="{2 * $page-width}"
              height="{$page-height}"
              page-number="{$sheet-number}"
              page-id="{$sheet-number}">
              
      <xsl:apply-templates select="xep:page[@page-number = $left-page]/*"/>
      <xep:translate y="0" x="{$page-width}"/>
      <xsl:apply-templates select="xep:page[@page-number = $right-page]/*"/>
      <xep:translate y="0" x="{-1 * $page-width}"/>
    </xep:page>      
  </xsl:template>

  <!-- ======================================================== -->
  <!-- Filter out interactive part (hot-areas): they won't work -->
  <!-- ======================================================== -->
  <xsl:template match="xep:external-link | xep:internal-link"/>

  <!-- =============================== -->
  <!-- Default identity transformation -->
  <!-- =============================== -->
  <xsl:template match="* | @* | text()" priority="-1">
    <xsl:copy>
      <xsl:apply-templates select="* | @* | text()"/>
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>